/**********************************************************************
* HEADER_START                                                        *
*                                                                     *
*      Name:           MAIN.C                                         *
*      Project:        Developper's HC08 Bootloader Master            *
*      Description:    MAIN file                                      *
*      Platform:       Linux/Win32                                    *
*      Revision:       1.0                                            *
*      Date:           Apr 8 2002                                     *
*      Author:         Michal Hanak                                   *
*      Company:        Motorola SPS                                   *
*      Security:       General Business                               *
*                                                                     *
* =================================================================== *
* Copyright (c):      MOTOROLA Inc., 2001, All rights reserved.       *
*                                                                     *
* =================================================================== *
* THIS SOFTWARE IS PROVIDED BY MOTOROLA "AS IS" AND ANY               *
* EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR  *
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL MOTOROLA OR              *
* ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    *
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT        *
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        *
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)            *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                  *
* =================================================================== *
*
* HEADER_END
*/

#include "sysdep.h"
#include "hc08sprg.h"
#include "serial.h"

////////////////////////////////////////////////
// global vars

IDENT_DATA ident;
BOARD_MEM  image;
int bl_version;

////////////////////////////////////////////////

int main(int argc, char *argv[ ])
{
	if(argc != 3 && argc != 4)
	{
		printf("usage: hc08sprg port[:][S|D|?] [speed] file\n"
			   "       port:D ... dual wire mode (default)\n"
			   "       port:S ... single wire mode\n"
			   "       port:? ... detect single/dual wire mode\n"
			   "       speed  ... speed in kbps\n"
			   "       file   ... S19 file\n"
			);
		exit(0);
	}

	// open serial port (speed specified ?)
	if(init_uart(argv[1], argc==4 ? argv[2] : NULL) < 0)
		exit(1);

	// read S19 file
	if(read_s19(argv[argc-1]) < 0)
		exit(2);

	// wait for reset
	if(hook_reset() < 0) 
		exit(3);

	// calibrate target's oscilator
	if(calibrate_speed() < 0)
		exit(4);

	// read info
	if(!read_mcu_info())
	{
		unsigned i;
		printf("Bootloader version string: %s\n", ident.targ_name);
		printf("Available flash memory: 0x%04X-0x%04X\n", ident.mem_start, ident.mem_end-1);
		printf("Erase block size: %3d bytes\n", ident.erblk);
		printf("Write block size: %3d bytes\n", ident.wrblk);
		printf("Original vector table: 0x%04X \n",ident.int_vect_tbl);
		printf("Bootloader user table: 0x%04X \n",ident.bl_tbl);
		printf("Bootloader data (hex): ");
		for(i=0; i<sizeof(ident.priv_data); i++)
			printf("%02x ", ident.priv_data[i]);
		printf("\n");
	}
	else 
	{
		fprintf(stderr, "Can't read MCU info (forgot to set single wire mode?)\n");
		exit(5);
	}
	
	// masquerade vector table
	if(setup_vect_tbl() < 0)
		exit(6);

	// check if code goes to valid memory only
//	if(
	check_image();
//	 < 0)
//		exit(7);

	// last bless
	printf("Are you sure to program part? [y/N]: ");
	if(toupper(getchar()) != 'Y') exit(-1);

	// program it!
	if(prg_mem() < 0)
	{
		fprintf(stderr, "Program memory failed.\n");
		exit(8);
	}

	// exit program mode, start application
	unhook();

	close_uart();
	return 0;
}
